#include	"lr2.h"


/*	<< (stream output) operator for the int16 type

	This operator currently always outputs the two bytes of the 16-bit value
	in Big-Endian order: most significant byte first.  As a result, output
	files will use the byte order that the Macintosh version of the LR2 Editor
	saves files with.  As far as I know, the Windows version of LR2 can read
	level files in Big-Endian (Mac) byte order. (PRX files appear to use 
	Little-Endian order on both machines?)  Eventually, it would be nice to include 
	byte-swapping logic so that my programs can output Little-Endian byte order 
	files on Windows.  
	
	Howver, simply swapping the output order in this function WILL NOT output
	correct Little-Endian order files.  This is because other parts of my LR2 
	programs break up the data structures into 2-byte pieces, corresponding to 
	my original assumption that most values were 16-bits wide.  I now have some
	evidence that values in level files vary between 1 and at least 6 bytes wide.
	So, a complete solution to the byte order issue would be significantly more
	complex and would require many changes throughout these programs.  For now,
	it is easiest to stick with outputting Big-Endian files.
	
	Anthony Kozar
	Jan. 6, 2012
 */	
std::ostream& operator<<( std::ostream& sout, int16 value )
{
	int8	highbyte, lowbyte;
	unsigned short val = (unsigned short)(value);
	
	lowbyte = int8(val & 0x00FF);
	val >>= 8;
	highbyte = int8(val & 0x00FF);
	
	sout	<< highbyte << lowbyte;
	return sout;
}
