/*	lr2.h

	Constants and other definitions for Lode Runner 2 programs.
	
	Anthony Kozar
	December 17, 2000
	
	June 17, 2003 - began working on LR2 level creators again.
	              - changed char to int8 for data (want to migrate to int16 eventually).
	              - added multiple-definition guards.
	              - began rewriting data using int16 type and creating enumeration constants.
	              
 */

#ifndef	ANTHONY_LR2_H
#define	ANTHONY_LR2_H

#include	<cstdlib>
#include	<iostream>

typedef	/*unsigned*/ char	int8;
//typedef	unsigned short	int16;
typedef	unsigned long	int32;

class	int16	{
	public:
		int16()				{ _val = 0; }
		int16(unsigned short value)	{ _val = value; }
		//~int16();
		
		operator unsigned short()	{ return _val; }
		
		//int16	operator=(const int16& value);
		int16	operator+(const int16& addend);
		int16	operator-(const int16& subtrahend);
		int16	operator*(const int16& multiplier);
		int16	operator/(const int16& divisor);
		
		bool	operator==(const int16& value)	{ return this->_val == value._val; }
		bool	operator!=(const int16& value)	{ return this->_val != value._val; }
		bool	operator<(const int16& value)		{ return this->_val < value._val; }
		bool	operator>(const int16& value)		{ return this->_val > value._val; }
		bool	operator<=(const int16& value)	{ return this->_val <= value._val; }
		bool	operator>=(const int16& value)	{ return this->_val >= value._val; }
		
	private:
		unsigned short	_val;
		unsigned short	overflow(long value)	{ return (value%65536); }
};

std::ostream& operator<<( std::ostream& sout, int16 value );

class	block_t {

};


/*	DATA    */

namespace	lr2	{


/*	union int16 {
		unsigned short;
		struct {
			int8	first;
			int8	second;
		};
	};
*/	
	// world constants
	typedef	int16	world_t;
	
	const	world_t	kMonaWorld		= 0x0000;
	const	world_t	kWackyWorld		= 0x0001;		// must use wacky header below?
	const	world_t	kIndustyWorld	= 0x0002;
	const	world_t	kAlienWorld		= 0x0002;		// not implemented
	const	world_t	kJungleWorld	= 0x0004;
	const	world_t	kBuilderWorld	= 0x0005;
	const	world_t	kGearWorld		= 0x0006;
	const	world_t	kMallWorld		= 0x0007;		// also not implemented
	const	world_t	kHubWorld		= 0x0008;
	const	world_t	kCreditsWorld	= 0x0009;

	
	/* Data for the level file header */
	const	int32		LEVEL_FILE_ID		= 0xD87E1B17;
	const	std::size_t	H1_SIZE			= 7;
	const	int16		HEADER_PART_1[H1_SIZE]	= { 0x0097, 0x0032, 0x0083, 0x0032, 0x0097, 0x0046, 0x00C8 };
	const	int16		WACKY_HEADER[H1_SIZE]	= { 0x0098, 0x0032, 0x0084, 0x0032, 0x0098, 0x0046, 0x00C8 };
				// in some Wacky levels:  { 0x0099, 0x0032, 0x0085, 0x0032, 0x0099, 0x0046, 0x00C8 };  // why?
	// the next three int16's are variable:
	// world_t	the level's world (e.g.  0x0005 = builder world, 0x0000 = mona world, etc.)
	// int16	0x0000
	// int16	number for the background image, these vary from 0x2af8 to 0x2b0f
	const	std::size_t	H2_SIZE			= 5;
	// the 2nd part of the header is all zeros UNLESS scrolling is locked
	const	int16		HEADER_PART_2[H2_SIZE]	= { 0x0000, 0x0000, 0x0000, 0x0000, 0x0000 };

	/* Item block data */
	// the item block starts with an int16 equal to the number of items followed by 0x0003
	// then a list of items in the form 0x00AD type FFFF Xcor Ycor Zcor, except for bombs
	// which have the form 0x00A7 000F bomb-type Xcor Ycor Zcor number (number can be 1-6),
	// and entry points which have the form 0x00A2 type FFFF Xcor Ycor Zcor.
	 
	// item type constants 
	typedef	int16	item_t;
	typedef	int16	bomb_t;
	
	const	item_t	kGold			= 0x0001;
	const	item_t	kLREntryPoint	= 0x0002;
	const	item_t	kMonkEntryPoint	= 0x0003;	
	const	item_t	kStasisMine		= 0x0008, 	kClock = 0x0008;
	const	item_t	kMorph		= 0x0009;
	const	item_t	kCloak		= 0x000A;
	const	item_t	kOneUp		= 0x000B;
	const	item_t	kGasCan		= 0x000C;
	const	item_t	kLandMines		= 0x000E, 	kMines = 0x000E;
	const	item_t	kBomb			= 0x000F;
	const	item_t	kDetonator		= 0x0011;
	const	item_t	kInviso		= 0x0012;
	const	item_t	kBeachBall		= 0x0013;
	const	item_t	kDonut		= 0x0014;
	
	// bomb type constants
	const	bomb_t	kSphere		= 0x0001;
	const	bomb_t	kXBomb		= 0x0002;
	const	bomb_t	kYBomb		= 0x0003;
	const	bomb_t	kCross		= 0x0004;
	const	bomb_t	kZBomb		= 0x0005;
	const	bomb_t	kTrinity		= 0x0006;


	/* Layout data */
	const	int		kMinimumX		= 10;
	const	int		kMaximumX		= 210;
	const	int		kMinimumY		= 10;
	const	int		kMaximumY		= 210;
	const	int		kMinimumZ		= 10;
	const	int		kMaximumZ		= 210;
	
	const	std::size_t	B1_SIZE				= 4;
	const	int16		DIGGABLE_BLOCK1[B1_SIZE]	= { 0x006E, 0x0001, 0x0000, 0x1C11 };	// block coordinates are next
	const	std::size_t	B2_SIZE				= 7;
	const	int16		DIGGABLE_BLOCK2[B2_SIZE]	= { 0x0001, 0x0000, 0x008C, 0x0000, 0x001C, 0x1900, 0x0000};	// block code is left off at end
	
	const	std::size_t	E1_SIZE				= 4;
	const	int16		EXIT_BLOCK1[E1_SIZE]	= { 0x0070, 0x0005, 0x0000, 0x1D14 };	// block coordinates are next
	const	std::size_t	E2_SIZE				= 8; // 9;
	const	int16		EXIT_BLOCK2[E2_SIZE]	= { 0x0001, 0x0000, 0x008F, 0x0000, 0x005D, 0x8D00, 0x0000, 0x2EE0/*, 0x0007*/ };	// block code is partially included
	
	const	std::size_t	INV1_SIZE				= 4;
	const	int16		INVISIBLE_BLOCK1[INV1_SIZE]	= { 0x006E, 0x0001, 0x0000, 0x0110 };	// block coordinates are next
	const	std::size_t	INV2_SIZE				= 8;
	const	int16		INVISIBLE_BLOCK2[INV2_SIZE]	= { 0x0001, 0x0000, 0x008C, 0x0000, 0x0001, 0x0900, 0x0000, 0x29E0};
	
	/* Block codes */
	
	// exits
	const int16		kBCMonaExit		= 0x0002;
	const int16		kBCWackyExit	= 0x0003;
	const int16		kBCIndustryExit	= 0x0004;
	const int16		kBCJungleExit	= 0x0006;
	const int16		kBCBuilderExit	= 0x0007;
	const int16		kBCGearExit		= 0x0008;
	const int16		kBCCreditsExit	= 0x000B;
	
	// Builder world
	const	int16		kBCBuilderBrown	= 0x1388;			// basic brown "builder world" block
	const	int16		kBCBuilderBlue	= 0x1389;			// basic blue "builder world" block
	
	// Wacky world
	const int16		kBCOrangeSliceMist	= 0x0BFE;		// orange slice with mist
	const int16		kBCLemonSliceMist		= 0x0BFF;		// lemon slice with mist
	const int16		kBCLimeSliceMist 		= 0x0C00;		// lime slice with mist
	const int16		kBCLimeSlice 		= 0x0C01;		// lime slice
	const int16		kBCLemonSlice 		= 0x0C02;		// lemon slice
	const int16		kBCOrangeSlice 		= 0x0C03;		// orange slice

	// Mona world
	const	int16		kBCMonaFlatStone		= 0x0439;		// mona world flat block
	
	// Jungle world
	const	int16		kBCJungleFlatStone	= 0x07DE;		// flat stone block
	const	int16		kBCJungleStoneVeg1	= 0x07DF;		// flat stone with vegatation #1
	const	int16		kBCJungleStoneVeg2	= 0x07E0;		// flat stone with vegatation #2
	
	// Gear world
	const	int16		kBCGearTranslucent	= 0x0420;		// "thin", blue, translucent tiles
	
	
/*	OLD DATA IN SINGLE-BYTE/DECIMAL FORMAT

	// header data
	int8	header[36]	= { 216,126,27,23,0,151,0,50,0,131,0,50,0,151,0,70,
				0,200,0,5,0,0,42,248,0,0,0,0,0,0,0,0,0,0 };

	// monk data
	int8	monk_header[4] = { 0,0,0,5 }; 					// header - no monks

	// item data

	// simple block data
	int8	block1[8]	= { 0,110,0,1,0,0,28,17 };				// block coordinates are next
	int8	block2[14]	= { 0,1,0,0,0,140,0,0,0,28,25,0,0,0 };		// block type is left off at end
	 
	// exit block data
	int8	exit1[8]	= { 0,112,0,5,0,0,29,20 };				// coordinates come next
	int8	exit2[18]	= { 0,1,0,0,0,143,0,0,0,93,141,0,0,0,46,224,0,7 };
*/
}


#endif
